/*++

	This program is free software; you can redistribute it
	and/or modify it under the terms of the Shared Code
	License (SCL) as published by www.IntellectualHeaven.com;
	either version 1.0, or (at your option) any later version.
	You should have received a copy of the SCL 1.0 license,
	look for file license.txt, along with this software; if not,
	then please write to x_pankaj_x@intellectualheaven.com
	for an electronic copy of the same.

Module Name:

    YArchDecodeView.h

Module Description:

	Main view class for YahDecode. It also implements the
	other functionality like decoding a yahoo archived
	message etc.

Author:

    Pankaj Garg		2004-09-17
    
Revision History:

--*/

#ifndef _YARCHDECODEVIEW_H_
#define _YARCHDECODEVIEW_H_


// System Include Files!!!
#include <windows.h>
#include <commctrl.h>
#include <tchar.h>
#include <string>
#include <vector>


// Local Include Files!!!


//
// ANSI Unicode independent string
//
#ifdef _UNICODE
typedef std::wstring	tstring;
#else
typedef std::string		tstring;
#endif

enum YARCH_NODE_TYPE
{
	NODE_CATEGORY,
	NODE_LEAF_USER
};


//
// For tree nodes. The node type represents what information is stored
// and not all the members may be present in a node
//
typedef struct _ARCHIVE_TREE_DATA
{
	YARCH_NODE_TYPE		nodeType;
	tstring				userName;
	tstring				peerName;	// For Messages its the friend name, for conf. its host name
	tstring				archivePath;

}ARCHIVE_TREE_DATA;


// List of archive root data
typedef std::vector<ARCHIVE_TREE_DATA>		ARCHIVE_TREE_DATA_LIST;


//
// For right side list control entries. Contains file name for
// a specific archive file
//
typedef struct _ARCHIVE_FILE_DATA
{
	tstring		userName;
	tstring		profileName;
	tstring		peerName;	// For Messages its the friend name, for conf. its host name
	tstring		archiveFile;

}ARCHIVE_FILE_DATA;

typedef std::vector<ARCHIVE_FILE_DATA>		ARCHIVE_FILE_DATA_LIST;


//
// Yahoo Archive File Format
//
// Yahoo stores the archive files in a special
// format. I tried to decode the files. I will
// present the information from my analysis, but
// there is no gurantee that it will be 100% correct
//

//
// The whole archive file is divided into entries
// You can treat this file as an array of such entries
// One entry in such a file looks like:
//
// |<-4 Bytes-->||<---4 Bytes-->||<---4 Bytes-->||<---4 Bytes-->||<Length Bytes>||<--4 Bytes--->||<Length Bytes>|
// |------------||--------------||--------------||--------------||--------------||--------------||--------------|
// | time_t		|| Code			|| sent/received|| Length (1)	|| XOR'ed data  || Length (2)	|| Raw ascii	|
// | entry		|| 06-Inst Msg	|| 0 - Sent		|| for the msg	|| equal to the || for the peer	|| peer user	|
// |			|| 19-Conf Join	|| 1 - received	|| sent / recvd || length (1).	|| user name.	|| name.		|
// |			|| 1D-Conf Msg	||				||				|| Doesn't exist|| Zero for IM	|| Doesn't exist|
// |			|| 1B-Left Conf ||				|| Can be Zero	|| if length is || Non-Zero for || if length is |
// |			|| 1A-????		||				||				|| zero.		|| Conference	|| zero.		|
//


typedef unsigned int UINT32;

typedef enum _YAHOO_MSG_TYPE
{
	YAHOO_MSG_UNKNOWN,
	YAHOO_MSG_IM,
	YAHOO_MSG_CONF

}YAHOO_MSG_TYPE;

#pragma pack(push)
#pragma pack(1)
typedef struct _YAHOO_ARCH_MSG_HDR
{
	// UInt32
	UINT32	time;
	UINT32	code;		// 06 - IM, 0x20 IM Reply (sometime), 19 - Joined conf, 1D - Message, 1B - Left Conf
	UINT32	received;	// 0 - means sent by us, 1 - means by received

}YAHOO_ARCH_MSG_HDR;

typedef struct _YAHOO_ARCH_MSG_DATA
{
	~_YAHOO_ARCH_MSG_DATA()
	{
		if (data)
		{
			delete data;
		}

		if (peerName)
		{
			delete peerName;
		}
	}

	UINT32	dataLength;
	BYTE	*data;
	UINT32	peerNameLength;
	BYTE	*peerName;

}YAHOO_ARCH_MSG_DATA;

#pragma pack(pop)


//
// Our Application's Title
//

#define APPLICATION_TITLE			_T("YahDecode - Yahoo Message Archive Decoder")


//
// Unique child window identifiers
// They should be unique for one parent
//
#define IDC_CHILD_STATUS_BAR		20001
#define IDC_CHILD_TOOL_BAR			20002
#define IDC_CHILD_MESSAGE_LIST		20003
#define IDC_CHILD_USER_LIST			20004
#define IDC_CHILD_EDIT_BOX			20005
#define IDC_CHILD_TREE_VIEW			20006


//
// status bar related data
//

// Number of breaks in Status bar
#define STATUS_BAR_NUM_PARTS					4
#define STATUS_WIDTH_OF_FIRST_PART				75
#define STATUS_WIDTH_OF_SECOND_PART				66
#define STATUS_WIDTH_OF_THIRD_PART				176
#define STATUS_WIDTH_OF_FOURTH_PART				98


//
// List Control related data and data types
//
#define LISTCTRL_MAX_COLUMN_NAME_LENGTH		64

typedef struct _ListControlHeaderData
{
	int				width;
	TCHAR			title[LISTCTRL_MAX_COLUMN_NAME_LENGTH];

}ListControlHeaderData;



//
// Class to handle application's GUI processing
//
class CYArchDecode
{
public:
	//
    // Constructor/Destructor
	//
    CYArchDecode(HINSTANCE hInstance);
    virtual ~CYArchDecode();

	//
    // Functions
	//
    void InitInstance();

	void ExitInstance();

	void Refresh();

    LRESULT WndProc(
				HWND hwnd,
				UINT nMsg,
				WPARAM wParam,
				LPARAM lParam);

	HACCEL GetAccel();

	HWND GetMainWindow();


private:
    
    void CreateMainWindow();

	void UpdateMenus();

	bool HandleMenuCommand(WORD	inMenuCmd);

    int  CreateStatusBar();

	void UpdateStatusBarData(tstring &szText);

	void CreateToolBar();

	void AddToolbarButton(
			int			iBitmap,
			int			idCommand,
			BYTE		fsState,
			BYTE		fsStyle,
			DWORD_PTR	dwData,
			INT_PTR		iString);

	

	void CreateMessageListControl();
	void PopulateMessageListCtrl(ARCHIVE_TREE_DATA &archiveRootData);
	void PopulateMessageListCtrlWholeCategory(ARCHIVE_TREE_DATA &archiveRootData);

	void CreateUserListControl();
	void PopulateUserListControl();
	void EnableArchiveForYahooUser(LPTSTR userName, DWORD itemIndex, BOOL shouldEnable);

	void CreateMessageTextBox();
	void InitMessageTextString(tstring &editBoxTextString, tstring &inFileName);
	void FinalizeMessageTextString(tstring &ioEditBoxTextString);
	void FormatUserNameString(tstring &userNameString, bool	dispSelfName);
	void FormatDecodedString(tstring &decodedString);
	void SetMessageTextBoxText(LPTSTR inText);
	void ReplaceChar(tstring &ioText, TCHAR inChar, LPTSTR inReplacement);
	void RemoveSpecialRTFChar(tstring &ioText);

	void PopulateTextBoxData(ARCHIVE_FILE_DATA &archiveFileData);
	bool ReadYahooMsgData(HANDLE hFile, YAHOO_ARCH_MSG_DATA &archMsgData);
	bool ReadYahooLenDataPair(HANDLE	hFile, UINT32 &length, BYTE* &data);

	void CreateUserTreeControl();
	void AttachImageListToTree();
	void PopulateUserTree();
	void TravelWinUserProfilesForYahooUsersArchive(tstring &winUserProfileFolder);
	void AddAllUsersArchivesToTree(tstring &dirPath);
	void AddUserArchivesToTree(tstring &userName, tstring &userProfilePath);
	void AddCategorizedUserArchiveToTree(tstring &userName, HTREEITEM &hItem, tstring &archivePath);
	

    int SaveAppInformation();

    int LoadAppInformation(RECT *pRect);


    //
	// Handles for different windows controls that we use in our
	// application
	//
    HINSTANCE       m_hInstance;
    HWND            m_hwnd;

    HWND            m_hwndStatus;
	HWND			m_hwndToolBar;
	HWND			m_hwndUserTree;
	HWND			m_hwndUserList;
	HWND			m_hwndMessageList;
	HWND			m_hwndMessageTextBox;
	HACCEL			m_hAccel;
	
	
	//
	// User tree image list and image index
	//
	HIMAGELIST		m_hILUserTree;
	int				m_UserMainImageIndex;
	int				m_MessageImageIndex;
	int				m_AlertImageIndex;
	int				m_ConferenceImageIndex;
	int				m_MobileImageIndex;
	int				m_UserSingleImageIndex;

	//
	// User tree leaf node params and message list params
	//
	ARCHIVE_TREE_DATA_LIST		m_TreeNodeParams;
	ARCHIVE_FILE_DATA_LIST		m_MessageListNodeParams;

	//
	// Used for decoded message box
	//
	bool			m_RichEdit;
	HFONT			m_MessageBoxFont;

	bool			m_ShowTimestamp;
	bool			m_OldComctl32;

};


#endif
